@@ -1,27 +1,23 @@
 package Net::DNS::RR::DNSKEY;
 
-#
-# $Id: DNSKEY.pm 1584 2017-07-28 16:15:17Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1584 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: DNSKEY.pm 1814 2020-10-14 21:49:16Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::DNSKEY - DNS DNSKEY resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
 
-use constant BASE64 => defined eval 'require MIME::Base64';
+use constant BASE64 => defined eval { require MIME::Base64 };
 
 #
 # source: http://www.iana.org/assignments/dns-sec-alg-numbers
@@ -54,8 +50,9 @@ use constant BASE64 => defined eval 'require MIME::Base64';
 
 	my %algbyval = reverse @algbyname;
 
-	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W]//g; uc($_) }, @algbyname;
-	my %algbyname = @algrehash;    # work around broken cperl
+	foreach (@algbyname) { s/[\W_]//g; }			# strip non-alphanumerics
+	my @algrehash = map { /^\d/ ? ($_) x 3 : uc($_) } @algbyname;
+	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
 		my $arg = shift;
@@ -63,12 +60,12 @@ use constant BASE64 => defined eval 'require MIME::Base64';
 		$key =~ s/[\W_]//g;				# strip non-alphanumerics
 		my $val = $algbyname{$key};
 		return $val if defined $val;
-		return $key =~ /^\d/ ? $arg : croak "unknown algorithm $arg";
+		return $key =~ /^\d/ ? $arg : croak qq[unknown algorithm "$arg"];
 	}
 
 	sub _algbyval {
 		my $value = shift;
-		$algbyval{$value} || return $value;
+		return $algbyval{$value} || return $value;
 	}
 }
 
@@ -77,37 +74,41 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
-	my $keylength = $self->{rdlength} - 4;
-	@{$self}{qw(flags protocol algorithm keybin)} = unpack "\@$offset n C2 a$keylength", $$data;
+	my $rdata = substr $$data, $offset, $self->{rdlength};
+	$self->{keybin} = unpack '@4 a*', $rdata;
+	@{$self}{qw(flags protocol algorithm)} = unpack 'n C*', $rdata;
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless defined $self->{algorithm};
-	pack 'n C2 a*', @{$self}{qw(flags protocol algorithm keybin)};
+	return pack 'n C2 a*', @{$self}{qw(flags protocol algorithm keybin)};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless defined $self->{algorithm};
-	$self->_annotation( 'Key ID =', $self->keytag ) if $self->{algorithm};
+	my $algorithm = $self->{algorithm};
+	$self->_annotation( 'Key ID =', $self->keytag ) if $algorithm;
 	return $self->SUPER::_format_rdata() unless BASE64;
-	my @base64 = split /\s+/, MIME::Base64::encode( $self->{keybin} );
-	my @rdata = ( @{$self}{qw(flags protocol algorithm)}, @base64 );
+	my @param = ( @{$self}{qw(flags protocol)}, $algorithm );
+	my @rdata = ( @param, split /\s+/, MIME::Base64::encode( $self->{keybin} ) || '-' );
+	return @rdata;
 }
 
 
 sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->flags(shift);
+	my $flags = shift;		## avoid destruction by CDNSKEY algorithm(0)
 	$self->protocol(shift);
 	$self->algorithm(shift);
+	$self->flags($flags);
 	$self->key(@_);
+	return;
 }
 
 
@@ -117,6 +118,8 @@ sub _defaults {				## specify RR attribute default values
 	$self->algorithm(1);
 	$self->flags(256);
 	$self->protocol(3);
+	$self->keybin('');
+	return;
 }
 
 
@@ -124,37 +127,43 @@ sub flags {
 	my $self = shift;
 
 	$self->{flags} = 0 + shift if scalar @_;
-	$self->{flags} || 0;
+	return $self->{flags} || 0;
 }
 
 
 sub zone {
-	my $bit = 0x0100;
-	for ( shift->{flags} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
-		return $_ & $bit;
+	my $self = shift;
+	if ( scalar @_ ) {
+		for ( $self->{flags} ) {
+			$_ = 0x0100 | ( $_ || 0 );
+			$_ ^= 0x0100 unless shift;
+		}
 	}
+	return 0x0100 & ( $self->{flags} || 0 );
 }
 
 
 sub revoke {
-	my $bit = 0x0080;
-	for ( shift->{flags} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
-		return $_ & $bit;
+	my $self = shift;
+	if ( scalar @_ ) {
+		for ( $self->{flags} ) {
+			$_ = 0x0080 | ( $_ || 0 );
+			$_ ^= 0x0080 unless shift;
+		}
 	}
+	return 0x0080 & ( $self->{flags} || 0 );
 }
 
 
 sub sep {
-	my $bit = 0x0001;
-	for ( shift->{flags} ) {
-		my $set = $bit | ( $_ ||= 0 );
-		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
-		return $_ & $bit;
+	my $self = shift;
+	if ( scalar @_ ) {
+		for ( $self->{flags} ) {
+			$_ = 0x0001 | ( $_ || 0 );
+			$_ ^= 0x0001 unless shift;
+		}
 	}
+	return 0x0001 & ( $self->{flags} || 0 );
 }
 
 
@@ -162,7 +171,7 @@ sub protocol {
 	my $self = shift;
 
 	$self->{protocol} = 0 + shift if scalar @_;
-	$self->{protocol} || 0;
+	return $self->{protocol} || 0;
 }
 
 
@@ -171,20 +180,19 @@ sub algorithm {
 
 	unless ( ref($self) ) {		## class method or simple function
 		my $argn = pop;
-		return $argn =~ /[^0-9]/ ? _algbyname($argn) : _algbyval($argn);
+		return $argn =~ /\D/ ? _algbyname($argn) : _algbyval($argn);
 	}
 
 	return $self->{algorithm} unless defined $arg;
 	return _algbyval( $self->{algorithm} ) if uc($arg) eq 'MNEMONIC';
-	$self->{algorithm} = _algbyname($arg) || die _algbyname('')    # disallow algorithm(0)
+	return $self->{algorithm} = _algbyname($arg) || die _algbyname('')    # disallow algorithm(0)
 }
 
 
 sub key {
 	my $self = shift;
-
-	$self->keybin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
+	return MIME::Base64::encode( $self->keybin(), "" ) unless scalar @_;
+	return $self->keybin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -192,30 +200,30 @@ sub keybin {
 	my $self = shift;
 
 	$self->{keybin} = shift if scalar @_;
-	$self->{keybin} || "";
+	return $self->{keybin} || "";
 }
 
 
-sub publickey { &key; }
+sub publickey { return shift->key(@_); }
 
 
 sub privatekeyname {
 	my $self = shift;
 	my $name = $self->signame;
-	sprintf 'K%s+%03d+%05d.private', $name, $self->algorithm, $self->keytag;
+	return sprintf 'K%s+%03d+%05d.private', $name, $self->algorithm, $self->keytag;
 }
 
 
 sub signame {
 	my $self = shift;
-	my $name = lc $self->{owner}->fqdn;
+	return lc $self->{owner}->fqdn;
 }
 
 
 sub keylength {
 	my $self = shift;
 
-	my $keybin = $self->keybin || return undef;
+	my $keybin = $self->keybin || return;
 
 	local $_ = _algbyval( $self->{algorithm} );
 
@@ -238,7 +246,7 @@ sub keylength {
 		return ( $T << 6 ) + 512;
 	}
 
-	length($keybin) << 2;		## ECDSA / ECC-GOST
+	return length($keybin) << 2;	## ECDSA / EdDSA
 }
 
 
@@ -267,7 +275,7 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name DNSKEY flags protocol algorithm publickey');
+    $rr = Net::DNS::RR->new('name DNSKEY flags protocol algorithm publickey');
 
 =head1 DESCRIPTION
 
@@ -300,7 +308,7 @@ Unsigned 16-bit number representing Boolean flags.
 	...
  }
 
-Boolean Zone flag.
+Boolean ZONE flag.
 
 =back
 
@@ -314,7 +322,7 @@ Boolean Zone flag.
 	...
  }
 
-Boolean Revoke flag.
+Boolean REVOKE flag.
 
 =back
 
@@ -328,7 +336,7 @@ Boolean Revoke flag.
 	...
  }
 
-Boolean Secure Entry Point flag.
+Boolean Secure Entry Point (SEP) flag.
 
 =back
 
@@ -421,6 +429,7 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC4034, RFC3755
 
-L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
+L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>,
+L<DNSKEY Flags|http://www.iana.org/assignments/dnskey-flags>
 
 =cut
